// BasicComponent.cpp: implementation of the BasicComponent class.
//
//////////////////////////////////////////////////////////////////////

#include "BasicComponent.h"
#include "ComponentVariant.h"
#include "Writer.h"
#include "ComponentRegister.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BasicComponent::BasicComponent(
	ComponentRegister * reg,
	long id,
	const char * name,
	Direction dir,
	long dx,
	long dy,
	long dt)
{
	m_variantList = NULL;
	m_variantListLength = 0;

	m_register = reg;
	m_id = id;
	m_name = new char[strlen(name) + 1];
	assert(m_name != NULL);
	strcpy(m_name, name);
	m_dt = dt;

	long rdx;
	long rdy;
	Direction rDir;
	Component::reverse(dx, dy, dir, &rdx, &rdy, &rDir);

	long i;
	for (i = 0; i < 8; i++)
	{
		Component::localToGlobal(0, 0, (Direction) i, dx, dy, &m_dx[i], &m_dy[i]);
		m_dir[i] = Component::combine((Direction) i, dir);

		// calculate 'reverse' coordinates -
		// if the component starts at them, it ends at [0, 0] with direction (i)
		Component::localToGlobal(0, 0, (Direction) i, rdx, rdy, &m_rdx[i], &m_rdy[i]);
		m_rDir[i] = Component::subtract((Direction) i, dir);
	}
}

BasicComponent::~BasicComponent()
{
	deleteVariantList();

	VariantIterator it = m_variantQueue.begin();

	while(it != m_variantQueue.end())
	{
		delete (*it);
		it++;
	}
	delete [] m_name;
}

void BasicComponent::print()
{
	Writer::print(getName());
}

void BasicComponent::deleteVariantList()
{
	if (m_variantList != NULL)
	{
		delete [] m_variantList;
		m_variantList = NULL;
		m_variantListLength = 0;
	}
}

void BasicComponent::prepare(long period, long compression)
{
	deleteVariantList();

	VariantIterator it = m_variantQueue.begin();
	long i = 0;

	while(it != m_variantQueue.end())
	{
		if ((*it)->query(period, compression))
		{
			i++;
		}
		it++;
	}

	m_variantList = new ComponentVariant *[i];
	assert(m_variantList != NULL);

	it = m_variantQueue.begin();
	while(it != m_variantQueue.end())
	{
		if ((*it)->query(period, compression))
		{
			m_variantList[m_variantListLength] = (*it);
			m_variantListLength++;
		}
		it++;
	}
	// calculate sizes
	for (long d = 0; d < 8; d++)
	{
		long minLoX, minHiX, minLoY, minHiY;
		long maxLoX, maxHiX, maxLoY, maxHiY;

		if (m_variantListLength <= 0)
		{
			minLoX = 0;
			minHiX = 0;
			minLoY = 0;
			minHiY = 0;
			maxLoX = 0;
			maxHiX = 0;
			maxLoY = 0;
			maxHiY = 0;
		}
		else
		{
			m_variantList[0]->getSize((Direction) d, minLoX, minHiX, minLoY, minHiY);
			maxLoX = minLoX;
			maxHiX = minHiX;
			maxLoY = minLoY;
			maxHiY = minHiY;
			long i;
			for (i = 1; i < m_variantListLength; i++)
			{
				long loX, hiX, loY, hiY;
				m_variantList[i]->getSize((Direction) d, loX, hiX, loY, hiY);
				// correct the minimum bounding box
				if (loX > minLoX)
				{
					minLoX = loX;
				}
				if (hiX < minHiX)
				{
					minHiX = hiX;
				}
				if (loY > minLoY)
				{
					minLoY = loY;
				}
				if (hiY < minHiY)
				{
					minHiY = hiY;
				}
				// correct the maximum bounding box
				if (loX < maxLoX)
				{
					maxLoX = loX;
				}
				if (hiX > maxHiX)
				{
					maxHiX = hiX;
				}
				if (loY < maxLoY)
				{
					maxLoY = loY;
				}
				if (hiY > maxHiY)
				{
					maxHiY = hiY;
				}
			}
		}
		// save the values
		m_minLoX[d] = minLoX;
		m_minHiX[d] = minHiX;
		m_minLoY[d] = minLoY;
		m_minHiY[d] = minHiY;

		m_maxLoX[d] = maxLoX;
		m_maxHiX[d] = maxHiX;
		m_maxLoY[d] = maxLoY;
		m_maxHiY[d] = maxHiY;
	}
}

void BasicComponent::deactivateVariant(ComponentVariant *var)
{
	long i;
	for (i = 0; i < m_variantListLength; i++)
	{
		if (m_variantList[i] == var)
		{
			var->deactivate();
			i++;
			while (i < m_variantListLength)
			{
				m_variantList[i - 1] = m_variantList[i];
				i++;
			}
			--m_variantListLength;
			return;
		}
	}
}


void BasicComponent::getMinimumSize(long &xMin, long &xMax, long &yMin, long &yMax)
{
	xMin = m_minLoX[TF];
	xMax = m_minHiX[TF];
	yMin = m_minLoY[TF];
	yMax = m_minHiY[TF];
}

void BasicComponent::getMaximumSize(long &xMin, long &xMax, long &yMin, long &yMax)
{
	xMin = m_maxLoX[TF];
	xMax = m_maxHiX[TF];
	yMin = m_maxLoY[TF];
	yMax = m_maxHiY[TF];
}

void BasicComponent::getMinimumSize(Direction dir, long &xMin, long &xMax, long &yMin, long &yMax)
{
	xMin = m_minLoX[dir];
	xMax = m_minHiX[dir];
	yMin = m_minLoY[dir];
	yMax = m_minHiY[dir];
}

void BasicComponent::getMaximumSize(Direction dir, long &xMin, long &xMax, long &yMin, long &yMax)
{
	xMin = m_maxLoX[dir];
	xMax = m_maxHiX[dir];
	yMin = m_maxLoY[dir];
	yMax = m_maxHiY[dir];
}



void BasicComponent::addVariant(ComponentVariant *var)
{
	m_variantQueue.push_back(var);
}
